package com.isu.gaswellwatch.aspect;

import cn.dev33.satoken.exception.NotLoginException;
import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.thread.threadlocal.NamedThreadLocal;
import com.isu.gaswellwatch.annotation.OperationLog;
import com.isu.gaswellwatch.config.SnowflakeConfig;
import com.isu.gaswellwatch.dto.LoginDTO;
import com.isu.gaswellwatch.dto.UserOperationRecordDTO;
import com.isu.gaswellwatch.entity.Response;
import com.isu.gaswellwatch.entity.ResponseCode;
import com.isu.gaswellwatch.entity.UserOperationRecord;
import com.isu.gaswellwatch.enums.LogResult;
import com.isu.gaswellwatch.enums.LogType;
import com.isu.gaswellwatch.service.UserOperationRecordService;
import com.isu.gaswellwatch.utils.ConverterUtil;
import com.isu.gaswellwatch.utils.IpUtils;
import com.isu.gaswellwatch.utils.ServletUtils;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.lang.reflect.Method;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

@Aspect
@Component
@Slf4j
public class OperationRecordAspect {

    @Autowired
    private SnowflakeConfig snowflakeConfig;
    @Autowired
    private UserOperationRecordService userOperationRecordService;
    @Autowired
    private HttpServletRequest request;

    private static final ThreadLocal<Date> beginTimeThreadLocal = new NamedThreadLocal<Date>("ThreadLocal beginTime");

    private UserOperationRecordDTO systemLog;

    @Pointcut("@annotation(com.isu.gaswellwatch.annotation.OperationLog)")
    public void operationRecordPointCut() {
    }

    @Before("operationRecordPointCut()")
    public void doBefore(JoinPoint joinPoint) throws Exception {
        systemLog= new UserOperationRecordDTO();
        //线程绑定变量（该数据只有当前请求的线程可见）
        Date beginTime = new Date();
        beginTimeThreadLocal.set(beginTime);
        //获取注解中的description属性值
        String description = getControllerMethodInfo(joinPoint).get("description").toString();
        Map<String, String> headers = ServletUtils.getHeaders(request);
        String menu = headers.get("menu");
        if (!StringUtils.isBlank(menu)) {
            systemLog.setMenu(URLDecoder.decode(menu, "utf-8"));
        }
        //日志标题
        systemLog.setName(description);
        //日志类型
        systemLog.setType((Integer) getControllerMethodInfo(joinPoint).get("type"));
        //日志请求url
        systemLog.setUri(request.getRequestURI());
        //请求方式
        systemLog.setRequestType(request.getMethod());
        //设置请求参数
        systemLog.setRequest(StringUtils.substring(Arrays.toString(joinPoint.getArgs()), 0, 1000));
        //其他属性
        systemLog.setIp(IpUtils.getIpAddr(request));
        systemLog.setStartTime(beginTime);
    }

    @Around("operationRecordPointCut()")
    public Object doAround(ProceedingJoinPoint proceedingJoinPoint) throws Throwable {
        Object proceed = null;
        try {
            proceed = proceedingJoinPoint.proceed();
            if (proceed == null) {
                systemLog.setResult(LogResult.ERROR.getVal());
                return proceed;
            }
            Response response = (Response) proceed;
            int code = response.getCode();
            if (code == ResponseCode.SUCCESS.code()) {
                systemLog.setResult(LogResult.SUCCESS.getVal());
            } else {
                systemLog.setResult(LogResult.ERROR.getVal());
                systemLog.setErrorMsg(StringUtils.substring(response.getMessage(), 0, 1000));
            }
            userOperationRecordService.addOperationRecord(ConverterUtil.convert(systemLog, UserOperationRecord.class));

        } catch (Exception e) {
            systemLog.setErrorMsg(StringUtils.substring(e.getMessage(), 0, 1000));
            systemLog.setResult(LogResult.ERROR.getVal());
            //入库缓存
            userOperationRecordService.addOperationRecord(ConverterUtil.convert(systemLog, UserOperationRecord.class));
            throw e;
        }finally {
            systemLog=null;
        }
        return proceed;
    }

    @After("operationRecordPointCut()")
    public void doAfter(JoinPoint joinPoint) throws Exception {
        Integer type = (Integer) getControllerMethodInfo(joinPoint).get("type");
        //请求开始时间
        long beginTime = beginTimeThreadLocal.get().getTime();
        //请求结束时间
        long endTime = System.currentTimeMillis();
        systemLog.setEndTime(new Date(endTime));
        //请求耗时
        Long logElapsedTime = endTime - beginTime;
        systemLog.setCostTime(logElapsedTime.intValue());
        systemLog.setId(snowflakeConfig.snowflakeId());
        String loginName = "";
        //用户登录获取登录名
        if (type.equals(LogType.LOGIN.getVal())) {
            Object[] args = joinPoint.getArgs();
            if (args != null && args.length > 0) {
                LoginDTO loginDTO = (LoginDTO) args[0];
                loginName = loginDTO.getUsername();
            }
        } else {
            //已登录用户从satoken获取
            try {
                loginName = StpUtil.getLoginIdAsString();

            } catch (NotLoginException e) {
                e.printStackTrace();
                loginName = "用户未登录";
            }
        }
        systemLog.setUsername(loginName);

    }


    /**
     * 获取注解中对方法的描述信息 用于Controller层注解
     *
     * @param joinPoint 切点
     * @return 方法描述
     * @throws Exception
     */
    public static Map<String, Object> getControllerMethodInfo(JoinPoint joinPoint) throws Exception {

        Map<String, Object> map = new HashMap<String, Object>(16);
        //获取目标类名
        String targetName = joinPoint.getTarget().getClass().getName();
        //获取方法名
        String methodName = joinPoint.getSignature().getName();
        //获取相关参数
        Object[] arguments = joinPoint.getArgs();
        //生成类对象
        Class targetClass = Class.forName(targetName);
        //获取该类中的方法
        Method[] methods = targetClass.getMethods();
        String description = "";
        Integer type = null;
        for (Method method : methods) {
            if (!method.getName().equals(methodName)) {
                continue;
            }
            Class[] clazzs = method.getParameterTypes();
            if (clazzs.length != arguments.length) {
                //比较方法中参数个数与从切点中获取的参数个数是否相同，原因是方法可以重载哦
                continue;
            }
            description = method.getAnnotation(OperationLog.class).description();
            type = method.getAnnotation(OperationLog.class).type().getVal();
            map.put("description", description);
            map.put("type", type);
        }
        return map;
    }


}
